package it.eng.spago.jbi.init;

import it.eng.spago.base.Constants;
import it.eng.spago.error.EMFErrorSeverity;
import it.eng.spago.error.EMFInternalError;
import it.eng.spago.jbi.component.ServiceUnitManagerProvider;
import it.eng.spago.tracing.TracerSingleton;

import java.util.HashMap;
import java.util.Set;

/**
 * Mantains the set of implementations for different JBI containers.
 * Implements methods useful to retrive an istance of 
 * a pool an to registry new connection pools  
 */
public class JBIManager {

	/**
	 * Set of containers.
	 */
	private static HashMap _containers = new HashMap();
	/**
	 * Name of the default repository
	 */
	private static String defaultContainer = null;

	/**
	 * Registers a new container pool  
	 * @param pool, the connection pool to register
	 * @param name, the name of the repository 
	 * @throws EMFInternalError Thrown if an error occurs during the registration
	 */
    protected static synchronized void registerContainer(ServiceUnitManagerProvider provider, String name) throws EMFInternalError {
    	if (provider == null) {
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.WARNING,
                                "JBIManager::registerRepository: jbi provider null");
            throw new EMFInternalError(EMFErrorSeverity.ERROR, "jbi provider null");
        }
    	if (name == null) {
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.WARNING,
                                "JBIManager::registerRepository: jbi provider name null");
            throw new EMFInternalError(EMFErrorSeverity.ERROR, "jbi provider name null");
        }
    	name = name.toUpperCase();
    	
	    if(_containers.keySet().contains(name)){
	       TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.WARNING,
                               "JBIManager::registerRepository: name already exists");
           throw new EMFInternalError(EMFErrorSeverity.ERROR, "Repository " + name + " already exists");
	    }
    	_containers.put(name, provider);
    }
	
    /**
     * Retrives an istance of the connection pool for
     * a specific repository identified by the inupt name
     * @param name, name of the repository 
     * @return Instance of the repository connection pool
     * @throws EMFInternalError Thrown if an error occurs during the retriving operation
     */
	public static synchronized ServiceUnitManagerProvider getInstance(String name) throws EMFInternalError{
    	if (name == null) {
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.WARNING,
                                "JBIManager::registerRepository: jbi provider name null");
            throw new EMFInternalError(EMFErrorSeverity.ERROR, "jbi provider name null");
        }
    	name = name.toUpperCase();

	    if(!_containers.keySet().contains(name)){
	       TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.WARNING,
                               "ContentRepositoryManager::getInstance: repository name not exists");
           throw new EMFInternalError(EMFErrorSeverity.ERROR, "Repository pool " + name + " non found");
	    }
	    else{
	    	return (ServiceUnitManagerProvider)_containers.get(name);
	    }
	}

	
	/**
	 * Checks if a pool with a given name is alrady registered
	 * @param name name of the pool
	 * @return boolean, true if the pool name is already registered, false otherwise
	 */
	public static synchronized boolean isRegistered(String name) {
		Set keys = _containers.keySet();
		return keys.contains(name);
	}
	
	
	
	/**
     * Destroy an istance of the connection pool for
     * a specific repository identified by the input name
     * @param name, name of the repository 
     * @throws EMFInternalError Thrown if an error occurs during the deletion
     */
	public static synchronized void destroyInstance(String name) throws EMFInternalError{
	    
		if(_containers.containsKey(name)) {
			_containers.remove(name);
		} else {
	       TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.WARNING,
                               "ContentRepositoryManager::destroyInstance: repository name not exists");
           throw new EMFInternalError(EMFErrorSeverity.ERROR, "Repository pool " + name + " non found");
	    }
	}
	
	
    /**
     * Retrives the istance of the connection pool for the default
     * repository
     * @return Instance of the repository connection pool
     * @throws EMFInternalError Thrown if an error occurs during the retriving operation
     */
	public static synchronized ServiceUnitManagerProvider getInstance() throws EMFInternalError{
		return getInstance(JBIManager.getDefaultContainer());
	}
	
	
	/**
     * Deletes an istance of the connection pool for the default repository
     * @throws EMFInternalError Thrown if an error occurs during the deletion
     */
	public static synchronized void destroyInstance() throws EMFInternalError{
		destroyInstance(JBIManager.getDefaultContainer());
	}
	
	/**
	 * Gets the name of the default repository
	 * @return name of the default repository
	 */
	public static String getDefaultContainer() {
		return defaultContainer;
	}

	/**
	 * Sets the name of the default repository
	 * @param defaultContainer, name to set
	 */
	protected static void setDefaultContainer(String defaultContainer) {
		JBIManager.defaultContainer = defaultContainer;
	}
}
